@@ -3,28 +3,29 @@
 
 from __future__ import unicode_literals
 
-from draftsman.classes.entity import Entity
-
 import copy
 import weakref
 
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:  # pragma: no coverage
+    from draftsman.classes.entity import Entity
+
 
 class Association(weakref.ref):
     """
-    A loose wrapper around weakref that permits deepcopying. Leads to better
-    memory management and better displaying.
+    A loose wrapper around weakref that permits deepcopying. Used to represent
+    wire and circuit connections, as well as associating train entities with
+    specific schedules. Leads to better memory management and better displaying.
     """
 
     def __init__(self, item):
         # type: (Entity) -> None
-        if not isinstance(item, Entity):
-            raise TypeError("cannot create association to a non-Entity")
+        # if not isinstance(item, Entity):
+        #     raise TypeError("cannot create association to a non-Entity")
 
         super(Association, self).__init__(item)
 
-    # def __copy__(self):
-    #     return self
-
     def __deepcopy__(self, memo):
         return self
 
@@ -44,10 +45,6 @@ class Association(weakref.ref):
         if self() is None:
             return "<Association to None>"
 
-        # return "<Association to {}{}>".format(
-        #     type(self()).__name__,
-        #     " with id '{}'".format(self().id) if self().id is not None else "",
-        # )
         return "<Association to {0} at 0x{1:016X}>".format(
             type(self()).__name__,
             id(self()),
