@@ -91,6 +91,7 @@ class BlueprintBook(object):
     of it's own metadata.
     """
 
+    @utils.reissue_warnings
     def __init__(self, blueprint_book=None):
         # type: (str, Union[str, dict]) -> None
         """
@@ -113,6 +114,7 @@ class BlueprintBook(object):
                 "string, a file object, a dictionary, or None"
             )
 
+    @utils.reissue_warnings
     def load_from_string(self, blueprint_string):
         # type: (str) -> None
         """
@@ -134,6 +136,7 @@ class BlueprintBook(object):
 
         self.setup(**root["blueprint_book"])
 
+    @utils.reissue_warnings
     def setup(self, **kwargs):
         # type: (**dict) -> None
         """
@@ -163,6 +166,9 @@ class BlueprintBook(object):
         if "label_color" in kwargs:
             self.label_color = kwargs.pop("label_color")
 
+        if "icons" in kwargs:
+            self.icons = kwargs.pop("icons")
+
         self.active_index = 0
         if "active_index" in kwargs:
             self.active_index = kwargs.pop("active_index")
