@@ -1,4 +1,7 @@
 # signals.py
+# -*- encoding: utf-8 -*-
+
+from __future__ import unicode_literals
 
 from draftsman import data
 from draftsman.error import InvalidSignalError
